/** 
 * A velocity controller for a single EyeBot wheel
 */

#include "DCVCtrl.h"
#include "PID.h"

/**
 * Velocity controller method implementations
 */

void DCVelControlInit(DCVelControl **ctrlRef,
		      DeviceSemantics m,
		      DeviceSemantics q) {

  *ctrlRef = (DCVelControl *)malloc(sizeof(DCVelControl));
  PIDInit(&((*ctrlRef)->pid));
  (*ctrlRef)->motor = MOTORInit(m);
  (*ctrlRef)->quad = QUADInit(q);
  QUADReset((*ctrlRef)->quad);
  (*ctrlRef)->pos = 0;
  (*ctrlRef)->timeScale = 1;
  (*ctrlRef)->driveRate = 0;
}

/**
 * Changes the desired wheel velocity (ticks/10 ms)
 */
void DCVelControlSet(DCVelControl *control, float velocity) {
	control->pid->setPoint = velocity;
}

/**
 * Returns the current setpoint of the velocity controller (ticks/10 ms)
 */
float DCVelControlGet(DCVelControl *control) {
	return control->pid->setPoint;
}

/**
 * Changes the time scale used in calculations: they are assumed
 * to occur with frequency 100/timeScale Hz
 */
void DCVelControlSetTimeScale(DCVelControl *control, int s) {
  control->timeScale = s;
}

/**
 * Changes the gains used in the velocity controller
 */
int DCVelControlSetGains(DCVelControl *control, float Kp, float Ki, float Kd) {
  return PIDSetGains(control->pid, Kp, Ki, Kd);
}

/**
 * Calculates a new input to MOTORDrive in order to
 * approach the desired velocity
 */
void DCVelControlCalc(DCVelControl *control) {
  int currPos;
  int dist;
  float vel;

  /* measure current position */
  currPos = QUADRead(control->quad);

  /* calculate velocity */
  dist = currPos - control->pos;
  vel = dist / control->timeScale;

  /* get new drive value from PID controller */
  control->driveRate += (int) PIDCalc(control->pid, vel);

  /* clip value to fit acceptable range */
  if (control->driveRate > 100) control->driveRate = 100;
  if (control->driveRate < -100) control->driveRate = -100;

  /* drive wheel */
  MOTORDrive(control->motor, control->driveRate);

  /* update position value */
  control->pos = currPos;
}

/**
 *	Releases the resources associated with the velocity controller
 */
void DCVelControlRelease(DCVelControl *control) {
  MOTORDrive(control->motor, 0);
  MOTORRelease(control->motor);
  QUADRelease(control->quad);
  PIDDestroy(control->pid);
  free(control);
}




